/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2012, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.as.test.integration.ws.wsrm;

import org.jboss.as.test.integration.ws.wsrm.generated.*;
import java.io.File;
import java.net.URL;
import javax.xml.namespace.QName;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.Service;

import org.jboss.logging.Logger;
import org.jboss.wsf.stack.cxf.client.UseNewBusFeature;

import javax.ejb.Startup;
import javax.ejb.Singleton;
import javax.ejb.Timer;
import javax.ejb.TimerConfig;
import javax.ejb.TimerService;
import javax.ejb.Timeout;
import javax.annotation.PostConstruct;
import javax.annotation.Resource;

/**
 * @author <a href="mailto:rsvoboda@redhat.com">Rostislav Svoboda</a>
 */
@Startup
@Singleton
public class ReliableServiceTestCase {

    private static final Logger log = Logger.getLogger(ReliableServiceTestCase.class);

    private static URL baseUrl = null;

    @Resource
    private TimerService timerService;

    public static void main(String[] args) {
        try {
            baseUrl = new URL("http://localhost:8080/ws-reliable-messaging-example/ReliableService");
            new ReliableServiceTestCase().runTests(); 
        } catch(Exception e) {
            e.printStackTrace();
        }
    }

    @PostConstruct
    public void initTimer() {
        timerService.createSingleActionTimer(5000, new TimerConfig(null, false));
    }
    @Timeout
    public void test() {
        try {
            baseUrl = new URL("http://localhost:8080/ws-reliable-messaging-example/ReliableService");
            new ReliableServiceTestCase().runTests(); 
        } catch(Exception e) {
            e.printStackTrace();
        }   
    }

    public void runTests() throws Exception {
        consumeHelloService();
        consumeOneWayService();
    }

    private void consumeOneWayService() throws Exception {
        QName serviceName = new QName("http://www.jboss.org/jbossws/ws-extensions/wsrm", "ReliableService");
        URL wsdlURL = new URL(baseUrl, "ReliableService?wsdl");
        System.out.println("wsdlURL: " + wsdlURL);
        //Service service = Service.create(wsdlURL, serviceName, new UseNewBusFeature());
        Service service = Service.create(wsdlURL, serviceName);
        ReliableService proxy = (ReliableService) service.getPort(ReliableService.class);

        BindingProvider bp = (BindingProvider) proxy;
        bp.getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, new URL(baseUrl, "ReliableService").toString());

        proxy.writeLogMessage();
    }

    private void consumeHelloService() throws Exception {
        QName serviceName = new QName("http://www.jboss.org/jbossws/ws-extensions/wsrm", "ReliableService");
        URL wsdlURL = new URL(baseUrl, "ReliableService?wsdl");
        System.out.println("wsdlURL: " + wsdlURL);
        //Service service = Service.create(wsdlURL, serviceName, new UseNewBusFeature());
        Service service = Service.create(wsdlURL, serviceName);
        ReliableService proxy = (ReliableService) service.getPort(ReliableService.class);

        BindingProvider bp = (BindingProvider) proxy;
        bp.getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, new URL(baseUrl, "ReliableService").toString());

        //Assert.assertEquals("Hello Rosta!", proxy.sayHello("Rosta"));
        System.out.println(proxy.sayHello("Rosta"));
    }
}
